<?php
// $Id: link_to_us.admin.inc,v 1.1.2.4 2010/07/24 21:17:26 alexiswilke Exp $

/**
 * @file
 * The administration forms.
 */

/**
 * \brief Actual implementation of hook_menu().
 *
 * This function create a list of items to add to
 * the system menu.
 *
 * \return The menu items.
 */
function _link_to_us_menu() {
  $items = array();

  // administration screens
  $items['admin/settings/link_to_us'] = array(
    'title' => 'Link to us',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('link_to_us_admin_settings'),
    'description' => 'Add a link_to_us link to pages.',
    'access arguments' => array('administer link_to_us'),
    'file' => 'link_to_us.admin.inc',
  );
  $items['admin/settings/link_to_us/list'] = array(
    'title' => 'Settings',
    'access arguments' => array('administer link_to_us'),
    'type' => MENU_DEFAULT_LOCAL_TASK,
    'weight' => -10,
    'file' => 'link_to_us.admin.inc',
  );

  $items['admin/settings/link_to_us/images'] = array(
    'title' => 'Images',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('link_to_us_images_form'),
    'access arguments' => array('administer link_to_us'),
    'type' => MENU_LOCAL_TASK,
    'file' => 'link_to_us.admin.inc',
  );

  // user page
  $items['link_to'] = array(
    'title' => 'Link to page',
    'page callback' => 'link_to_us_controller',
    'access arguments' => array('access content'),
    'type' => MENU_CALLBACK,
    'file' => 'link_to_us.page.inc',
  );

  return $items;
}

/**
 * \brief Administer the Link to us settings.
 *
 * Much of this banner upload and admin code was borrowed from the user_badges module.
 * Many thanks for paving the way.
 *
 * \return The settings form.
 */
function link_to_us_admin_settings() {
  // PAGE settings
  $form['link_to_us_page'] = array(
    '#type' => 'fieldset',
    '#title' => t('Page settings'),
    '#description' => t('To upload or delete images from your banner page, go <a href="@here">Link to us -&gt; images</a>.', array('@here' => url('admin/settings/link_to_us/images'))),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['link_to_us_page']['link_to_us_header'] = array(
    '#type' => 'textarea',
    '#title' => t('Link page header'),
    '#default_value' => variable_get('link_to_us_header', LINK_TO_US_HEADER),
    '#description' => t('This text will appear at the top of the <strong>Link to us</strong> page.'),
  );
  $form['link_to_us_page']['link_to_us_footer'] = array(
    '#type' => 'textarea',
    '#title' => t('Link page footer'),
    '#default_value' => variable_get('link_to_us_footer', LINK_TO_US_FOOTER),
    '#description' => t('This text will appear at the bottom of the <strong>Link to us</strong> page.'),
  );

  $form['link_to_us_page']['link_to_us_show_node_teaser'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show the node teaser'),
    '#default_value' => variable_get('link_to_us_show_node_teaser', TRUE),
    '#description' => t('When selected, show the node teaser at the top of the page. By default, the teaser is shown in a box.'),
  );

  $form['link_to_us_page']['link_to_us_textlink'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show the text only link'),
    '#default_value' => variable_get('link_to_us_textlink', FALSE),
    '#description' => t('Show an optional text only link. Will use the title of the node, category, or the site name as needed.'),
  );

  $form['link_to_us_page']['link_to_us_home_page'] = array(
    '#type' => 'checkbox',
    '#title' => t('Banners link to home page'),
    '#default_value' => variable_get('link_to_us_home_page', FALSE),
    '#description' => t('By default, the node link area has a link to the <em>Link to us page</em> that generates a link to that specific node. When this flag is set, the link always goes to your &lt;front&gt; page. It is suggested that you change the default link message when you use this option (see <em>Node link label</em> below.)'),
  );

  $form['link_to_us_page']['link_to_us_fieldset'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use a fieldset'),
    '#default_value' => variable_get('link_to_us_fieldset', FALSE),
    '#description' => t('Put the links in a fieldset.'),
  );

  $form['link_to_us_page']['link_to_us_max_width'] = array(
    '#type' => 'textfield',
    '#title' => t('Maximum width'),
    '#default_value' => variable_get('link_to_us_max_width', 0),
    '#description' => t('Specify a width, in pixels, for the box holding the text link and banners. Use 0 to not force the width. The minimum width used is 150 pixels.'),
  );

  $form['link_to_us_page']['link_to_us_textarea'] = array(
    '#type' => 'textfield',
    '#title' => t('Textarea sizes'),
    '#default_value' => variable_get('link_to_us_textarea', '60x3'),
    '#description' => t('The sizes of the text area to hold the link for the user (i.e. 60x3). For a resizable textarea, add \'r\' at the end (i.e. 60x3r). That area is read-only and when the height is set to 1, it is transformed to a textfield.'),
  );

  $form['link_to_us_page']['link_to_us_text_help'] = array(
    '#type' => 'textfield',
    '#title' => t('Help about the text link'),
    '#default_value' => variable_get('link_to_us_banner_help', LINK_TO_US_TEXT_HELP),
    '#description' => t('This string is shown right before the text link that the user can copy &amp; paste on their website. Use a space to hide this message. Remember that the text link is hidden by default, you must activate it below so it works. Use &lt;none&gt; to hide this label.'),
  );

  $form['link_to_us_page']['link_to_us_banner_help'] = array(
    '#type' => 'textfield',
    '#title' => t('Help about each banner link'),
    '#default_value' => variable_get('link_to_us_banner_help', LINK_TO_US_BANNER_HELP),
    '#description' => t('This string is shown right before each string that the user can copy &amp; paste on their website. Use a space to hide this message. Use &lt;none&gt; to hide this label.'),
  );


  // LINK settings
  $form['link_to_us_link'] = array(
    '#type' => 'fieldset',
    '#title' => t('Link settings'),
    '#description' => t('Setup the link that appears at the bottom of each page.'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['link_to_us_link']['link_to_us_teaser'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show link on teasers'),
    '#default_value' => variable_get('link_to_us_teaser', FALSE),
    '#description' => t('When checked, show the links on pages and teasers. Otherwise, only pages receive the link.'),
  );

  $tokens = t('HTML okay, which means you want to write &amp; as &amp;amp; and &lt; as &amp;lt; and &gt; as &amp;gt;. Tokens available: @page, @type, @title, @nid.');
  $form['link_to_us_link']['link_to_us_link_label'] = array(
    '#type' => 'textfield',
    '#title' => t('Node link label'),
    '#default_value' => variable_get('link_to_us_link_label', LINK_TO_US_LINK_LABEL),
    '#description' => t('This string is used as the label of the link added at the bottom of each page.')
      . '<br />' . $tokens,
  );
  $form['link_to_us_link']['link_to_us_link_title'] = array(
    '#type' => 'textfield',
    '#title' => t('Node link title'),
    '#default_value' => variable_get('link_to_us_link_title', LINK_TO_US_LINK_TITLE),
    '#description' => t('This string is used as the title of the link added at the bottom of each page.')
      . '<br />' . $tokens,
  );

  $form['link_to_us_link']['link_to_us_nodes'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Node Types'),
    '#default_value' => variable_get('link_to_us_nodes', array()),
    '#options' => array_map('check_plain', node_get_types('names')),
    '#description' => t('Node types that will receive the <strong>Link to us</strong> link.'),
  );

  return system_settings_form($form);
}

/**
 * \brief Form allowing administrators to add banner images.
 *
 * Define a form to upload banner images.
 */
function link_to_us_images_form() {
  if (!user_access('upload file')) {
    drupal_set_message(t('Sorry! You do not have the right to upload file. Please, contact your administrator so he can enable that feature for you.'), 'error');
    return drupal_access_denied();
  }

  $form = array('#skip_duplicate_check' => TRUE);
  if (module_exists('upload')) {
    $form['new']['banner'] = array('#type' => 'file', '#title' => t('Upload image'), '#size' => 40);
    $form['new']['attach'] = array('#type' => 'submit', '#value' => t('Upload'));
  }
  else {
    drupal_set_message(t('Upload of images requires the upload module to be enabled. !link',
      array('!link' => l(t('Enable modules here.'), 'admin/build/modules'))), 'error');
  }

  $form['#attributes']['enctype'] = 'multipart/form-data';

  $selects = link_to_us_image_selects();
  if (count($selects)) {
    $form['images'] = array('#tree' => TRUE);
    foreach ($selects as $imagepath => $imageimg) {
      $form['images'][$imagepath] = array(
        '#type' => 'checkbox',
        '#title' => $imageimg,
        '#return_value' => 1,
        '#default_value' => FALSE,
        '#description' => check_plain($imagepath),
      );
    }
    $form['delete_image'] = array(
      '#type' => 'submit',
      '#value' => t('Delete'),
    );
  }

  return $form;
}

/**
 * \brief Validate the images submission.
 *
 * Check whether:
 *
 * \li Delete has been chosen AND a checkbox has been selected.
 * \li Upload has been chosen AND the file upload form is not empty.
 *
 * \param[in] $form The current form.
 * \param[in,out] $form_state The current form state.
 */
function link_to_us_images_form_validate($form, &$form_state) {
  if ($form_state['values']['op'] == t('Upload')) {
    // XXX as far as I can tell, there isn't a way to check that with an API
    if (empty($_FILES['files']['name']['banner'])) {
      form_set_error('banner', t('Please enter the filename of an image to upload and try again.'));
    }
  }
  elseif ($form_state['values']['op'] == t('Delete')) {
    if (count(array_filter($form_state['values']['images'])) == 0) {
      form_set_error('images', t('Please select all the images to delete.'));
    }
  }
}

/**
 * \brief Implement extra submit actions.
 *
 * This function takes care of deleting files that were checked for deletion.
 *
 * \param[in] $form The current form.
 * \param[in,out] $form_state The current form state.
 */
function link_to_us_images_form_submit($form, &$form_state) {
  if ($form_state['values']['op'] == t('Upload')) {
    $dir = file_create_path('link_banners');
    $is_writable = file_check_directory($dir, 1);
    if ($is_writable) {
      global $user;

      $limits = _upload_file_limits($user);
      $accepted_extensions = array('gif', 'jpg', 'jpeg', 'png');
      $ext = array_intersect(explode(' ', $limits['extensions']), $accepted_extensions);
      $validators = array(
        'file_validate_extensions' => $ext,
        'file_validate_is_image' => array(),
        'file_validate_image_resolution' => array($limits['resolution']),
        'file_validate_size' => array($limits['file_size'], $limits['user_size']),
      );
      if (user_access('upload files') && $file = file_save_upload('banner', $validators, $dir)) {
        file_set_status($file, FILE_STATUS_PERMANENT);
        drupal_set_message(t('New image saved.'));
      }
    }
    else {
      drupal_set_message('The directory @dir is not writable by your web server.',
        array('@dir' => $dir));
    }
  }
  elseif ($form_state['values']['op'] == t('Delete')) {
    $to_delete = array();
    foreach ($form_state['values']['images'] as $path => $do_remove) {
      if ($do_remove) {
        $to_delete[] = $path;
      }
    }
    if (count($to_delete) > 0) {
      link_to_us_image_delete($to_delete);
    }
  }
}

/**
 * \brief Remove a list of file.
 *
 * \param[in] $to_delete An array of filename to be deleted.
 */
function link_to_us_image_delete($to_delete) {
  $dir = file_create_path('link_banners');
  foreach ($to_delete as $path) {
    if (file_check_location($path, $dir)) {
      file_delete($path);
    }
  }
}

// vim: ts=2 sw=2 et syntax=php
